from ewokscore.cliutils import add_execute_parameters as _add_execute_parameters
from ewokscore.cliutils import apply_execute_parameters
from ewokscore.cliutils import add_convert_parameters  # noqa F401
from ewokscore.cliutils import apply_convert_parameters  # noqa F401


def add_execute_parameters(parser, shell=False):
    _add_execute_parameters(parser, shell=shell)
    parser.add_argument(
        "--binding",
        type=str,
        choices=["none", "dask", "ppf", "orange"],
        default="none",
        help="Task binding to be used",
    )


def add_submit_parameters(parser, shell=False):
    add_execute_parameters(parser, shell=shell)
    parser.add_argument(
        "--wait",
        type=float,
        default=-1,
        help="Timeout for receiving the result (negative number to disable)",
    )


def apply_submit_parameters(args, shell=False):
    apply_execute_parameters(args, shell=shell)



import os
import importlib
from typing import Any, Optional, List, Union
from ewokscore.graph import TaskGraph
from ewokscore.events.contexts import job_context, RawExecInfoType


try:
    from ewoksjob.client import submit
except ImportError:
    submit = None


__all__ = ["execute_graph", "load_graph", "save_graph", "convert_graph", "submit_graph"]


def import_binding(binding: Optional[str]):
    if not binding or binding.lower() == "none":
        binding = "ewokscore"
    elif not binding.startswith("ewoks"):
        binding = "ewoks" + binding
    return importlib.import_module(binding)


def execute_graph(
    graph,
    binding: Optional[str] = None,
    inputs: Optional[List[dict]] = None,
    load_options: Optional[dict] = None,
    execinfo: RawExecInfoType = None,
    environment: Optional[dict] = None,
    **execute_options
):
    with job_context(execinfo, binding=binding) as execinfo:
        if load_options is None:
            load_options = dict()
        if environment:
            environment = {k: str(v) for k, v in environment.items()}
            os.environ.update(environment)
        graph = load_graph(graph, inputs=inputs, **load_options)
        mod = import_binding(binding)
        return mod.execute_graph(graph, execinfo=execinfo, **execute_options)


def submit_graph(graph, **options):
    if submit is None:
        raise RuntimeError("requires the 'ewoksjob' package")
    return submit(args=(graph,), kwargs=options)


def load_graph(
    graph: Any, inputs: Optional[List[dict]] = None, **load_options
) -> TaskGraph:
    binding = _get_binding_for_format(graph, options=load_options)
    mod = import_binding(binding)
    return mod.load_graph(graph, inputs=inputs, **load_options)


def save_graph(graph: TaskGraph, destination, **save_options) -> Union[str, dict]:
    binding = _get_binding_for_format(destination, options=save_options)
    mod = import_binding(binding)
    return mod.save_graph(graph, destination, **save_options)


def convert_graph(
    source,
    destination,
    inputs: Optional[List[dict]] = None,
    load_options: Optional[dict] = None,
    save_options: Optional[dict] = None,
) -> Union[str, dict]:
    if load_options is None:
        load_options = dict()
    if save_options is None:
        save_options = dict()
    graph = load_graph(source, inputs=inputs, **load_options)
    return save_graph(graph, destination, **save_options)


def _get_binding_for_format(graph, options: Optional[dict] = None) -> Optional[str]:
    """Get the binding which implements the workflow format (loading and saving)."""
    representation = None
    if options:
        representation = options.get("representation")
    if (
        representation is None
        and isinstance(graph, str)
        and graph.lower().endswith(".ows")
    ):
        representation = "ows"
    if representation == "ows":
        return "orange"



import os
import sys
import pytest

from ewoks.__main__ import main
from ewokscore import load_graph
from ewokscore.tests.examples.graphs import graph_names
from ewokscore.tests.examples.graphs import get_graph
from ewokscore.tests.utils.results import assert_execute_graph_default_result


@pytest.mark.parametrize("graph_name", graph_names())
@pytest.mark.parametrize("scheme", (None, "json"))
@pytest.mark.parametrize("binding", (None, "dask", "ppf"))
def test_execute(graph_name, scheme, binding, tmpdir):
    graph, expected = get_graph(graph_name)
    argv = [
        sys.executable,
        "execute",
        graph_name,
        "--test",
        "--output",
        "all",
        "--merge-outputs",
    ]
    if binding:
        argv += ["--binding", binding]
    if scheme:
        argv += ["--data-root-uri", str(tmpdir), "--data-scheme", scheme]
        varinfo = {"root_uri": str(tmpdir), "scheme": scheme}
    else:
        varinfo = None

    keep = graph
    ewoksgraph = load_graph(graph)
    non_dag = ewoksgraph.is_cyclic or ewoksgraph.has_conditional_links

    if non_dag and binding != "ppf":
        with pytest.raises(RuntimeError):
            main(argv=argv, shell=False)
        return

    result = main(argv=argv, shell=False)

    assert_execute_graph_default_result(ewoksgraph, result, expected, varinfo)
    assert keep == graph


@pytest.mark.parametrize("graph_name", graph_names())
def test_convert(graph_name, tmpdir):
    destination = str(tmpdir / f"{graph_name}.json")
    argv = [
        sys.executable,
        "convert",
        graph_name,
        destination,
        "--test",
        "-s",
        "indent=2",
    ]
    main(argv=argv, shell=False)
    assert os.path.exists(destination)



import sys
import argparse

from pprint import pformat
from . import cliutils
from .bindings import execute_graph
from .bindings import convert_graph
from .bindings import submit_graph


def create_argument_parser(shell=False):
    parser = argparse.ArgumentParser(
        description="Esrf WOrKflow Sytem CLI", prog="ewoks"
    )

    subparsers = parser.add_subparsers(help="Commands", dest="command")
    execute = subparsers.add_parser("execute", help="Execute a workflow")
    submit = subparsers.add_parser("submit", help="Schedule a workflow execution")
    convert = subparsers.add_parser("convert", help="Convert a workflow")
    cliutils.add_execute_parameters(execute, shell=shell)
    cliutils.add_submit_parameters(submit, shell=shell)
    cliutils.add_convert_parameters(convert, shell=shell)
    return parser


def command_execute(args, shell=False):
    cliutils.apply_execute_parameters(args, shell=shell)
    results = execute_graph(args.graph, binding=args.binding, **args.execute_options)
    print("Result of workflow '%s':\n%s" % (args.workflow, pformat(results)))

    if shell:
        if results is None:
            return 1
        else:
            return 0
    else:
        return results


def command_submit(args, shell=False):
    cliutils.apply_submit_parameters(args, shell=shell)
    future = submit_graph(args.graph, binding=args.binding, **args.execute_options)
    print(f"Job submitted (ID: {future.task_id})")
    if args.wait >= 0:
        print(future.get(timeout=args.wait))


def command_convert(args, shell=False):
    cliutils.apply_convert_parameters(args, shell=shell)
    convert_graph(args.graph, args.destination, **args.convert_options)


def command_default(args, shell=False):
    if shell:
        return 0
    else:
        return None


def main(argv=None, shell=True):
    parser = create_argument_parser(shell=shell)

    if argv is None:
        argv = sys.argv
    args = parser.parse_args(argv[1:])

    if args.command == "execute":
        return command_execute(args, shell=shell)
    elif args.command == "submit":
        return command_submit(args, shell=shell)
    elif args.command == "convert":
        return command_convert(args, shell=shell)
    else:
        parser.print_help()
        return command_default(args, shell=shell)


if __name__ == "__main__":
    sys.exit(main())



from .bindings import load_graph  # noqa: F401
from .bindings import save_graph  # noqa: F401
from .bindings import convert_graph  # noqa: F401
from .bindings import execute_graph  # noqa: F401

__version__ = "0.1.1"
